﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace ML_DSO
{
    public enum MPResult
    {
        MPUSB_FAIL = 0,
        MPUSB_SUCCESS = 1,
        MPUSB_SUCCESS_LESS_DATA = 2
        //TimeOut = 2 in case of ethernet
    }

    public enum E_MEAS
    {
        TopMeasurement = 0x0,
        BaseMeasurement,
        MinMeasurment,
        MaxMeasurment,
        RiseTimeEye,
        FallTimeEye,
        RiseTimeEyeStartX,
        RiseTimeEyeEndX,
        FallTimeEyeStartX,
        FallTimeEyeEndX,
        PeakToPeak,
        EyeAmplitude,
        EyeHeight,
        EyeHeightTop,
        EyeHeightBase,
        EyeWidth,
        JitterP2P,
        JitterStartX,
        JitterEndX,
        CrossingY,
        CrossingX,
        PointsPerUI,
        FailingPointsCount,
        MaskMargin,
        RJppByBER,
        DJ,
        RJrms,
        RJrmsMinIntrinsic,
        TotalJitterByBER,
        OneLevel,
        ZeroLevel,
        VoltageAmplitude,
        RiseTimePattern,
        FallTimePattern,
        PreEmphasisHeight,
        PreEmphasisHeightDB,
        PreEmphasisWidth,
        DeEmphasisHeight,
        DeEmphasisHeightDB,
        DeEmphasisWidth,
        Oma,
        OmaDBm,
        Aop,
        AopDBm,
        ExtinctionRatio,
        TotalJitterJ2,
        TotalJitterJ9,
        DN,
        RNrms,
        RNpp,
        TotalNoiseByBER,
        EyeHeightByBER,
        SNR,
        JitterRms,
        EyeWidthByBER,
        VerticalEyeClosure,
        ThreeLevel,
        TwoLevel,
        Pam4_AVupp,
        Pam4_AVmid,
        Pam4_AVlow,
        Pam4_Vupp,
        Pam4_Vmid,
        Pam4_Vlow,
        Pam4_Hupp,
        Pam4_Hmid,
        Pam4_Hlow,
        Pam4_VuppBER,
        Pam4_VmidBER,
        Pam4_VlowBER,
        Pam4_HuppBER,
        Pam4_HmidBER,
        Pam4_HlowBER,
        Pam4_Vupp_Top,
        Pam4_Vmid_Top,
        Pam4_Vlow_Top,
        Pam4_Hupp_StartX,
        Pam4_Hmid_StartX,
        Pam4_Hlow_StartX,
        Pam4_Vupp_Base,
        Pam4_Vmid_Base,
        Pam4_Vlow_Base,
        Pam4_Hupp_EndX,
        Pam4_Hmid_EndX,
        Pam4_Hlow_EndX
    };

    public enum Operation : int
    {
        Read,
        Write
    }

    public enum SignalCoding : int
    {
        NRZ = 1,
        PAM4 = 2,
    }


    public enum StatisticResult
    {
        Latest = 0,
        Average,
        Min,
        Max,
        StandardDeviation
    }


    public class DSO_Wrapper
    {
        #region Members

        public bool isConnected;
        public bool DSOisConfigured = false;
        const int e_pkt_len = 512;

        public void SetIsConnected(bool isConnected)
        {
            this.isConnected = isConnected;
        }

        public bool IsConnected()
        {
            return this.isConnected;
        }
        #endregion

        #region "Enumerations"

        enum E_DVT
        {
            FIFO_Reset,
            BootLoader_Mode,
            Port,
            EthIP_MSB,
            EthIP_LSB,
            EthMask_MSB,
            EthMask_LSB,
            EthGateway_MSB,
            EthGateway_LSB,
            DevFirmware_REv,
            EEProm_Adr,
            EEProm_Data,
            Model_Number,
            Serial_Number,
            Calibration_State,
            Calibration_Time,
            ML4016_TempSense,
            Fan_Speed,
            FPGA_PWDN,
            Low_PWR_Mode,
            Board_ID,
            Board_Name_1,
            Board_Name_2,
            Board_Name_3,
            Board_Name_4,
            Board_Name_5,
            EthMac_HSB,
            EthMac_MSB,
            EthMac_LSB,
            BW_Select
        };
        enum E_Clock
        {
            REF_CLK_Switch = 0x0100,
            Board_Pre_Clock_Divider,
            Board_Post_Clock_Divider,
            Trigger_Out_Sel,
            Chip_Ref_Clk_Sel,
            Chip_Ref_Clk_Div,
            Clock_Register_Adr,
            Clock_Register_Data,
            Clock_Status
        };


        #endregion

        #region structure
        public struct FWInfo
        {
            public int FWDeviceID;
            public int FWRev;
            public string DateAndTime;
        }

        #endregion

        byte instance = 0;

        #region Constructor
        public DSO_Wrapper(byte instance)
        {
            this.instance = instance;
        }
        #endregion

        static object connectionMonitor = new object();

#if DEBUG
        const string dllPath = @"lib\lib-debug\MLDSO-API.dll";
#else
        const string dllPath = @"lib\MLDSO-API.dll";
#endif

        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIConnectToDSO")]
        private static extern bool ConnectToDSO(byte instance, string dsoIP, bool isOpticalDevice, int opticalWavelength, string[] simulationFiles, int numSimFiles);

        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIConfigureApplication")]
        private static extern bool ConfigureApplication(byte instance, string saveConfig, string saveData, bool saveStatistics, bool saveDataFlag, bool statisticsPrint, bool eyePrint);

        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIConfigureDSO")]
        private static extern bool ConfigureDSO(byte instance,
            double lineRateInput, double packetCountInput, int patternLengthInput,
            bool wanderOn, bool precisionTimeBaseOn, bool notmalModeInput, double clockInput,
            double nrzWindowStartInput, double nrzWindowsEndInput,
            bool configureDSO, int samplingFrequencyIndexInput, double attenuatorCompensationDB,
            double totalJitterTargetBER, double deviceIntrinsicJitterFemtoSec, double dJCorrection = 1, double rJrmsCorrection = 1, bool isFilterPjAtSource = false, int PercentUIforlevels = 20);

        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIConfigureDSO")]
        private static extern bool ConfigureDSO(byte instance,
            double lineRateInput, double packetCountInput, int patternLengthInput,
            bool wanderOn, bool precisionTimeBaseOn, bool notmalModeInput, double clockInput,
            double nrzWindowStartInput, double nrzWindowsEndInput,
            bool configureDSO, int samplingFrequencyIndexInput, double attenuatorCompensationDB,
            double totalJitterTargetBER, double deviceIntrinsicJitterFemtoSec, double dJCorrection, double rJrmsCorrection, bool isFilterPjAtSource, int percentUIforLevel, ref double samplingFreq, int signalCoding);

        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIConfigureEyeMeasurements")]
        private static extern bool ConfigureEyeMeasurements(byte instance, bool doTopMeasurements, bool doBaseMeasurements, bool doMin,
                                      bool doMax, bool doRiseTime, bool doFallTime, bool doPeakToPeak,
                                      bool doEyeAmplitude, bool doEyeHeight, bool doEyeWidth, bool doCrossingPercentage, bool doJitter, bool doSNR, bool doVEC);

        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIConfigurePatternMeasurements")]
        private static extern bool ConfigurePatternMeasurements(byte instance,
                                          bool doRJ, bool doDJ, bool doRiseTimePatterm,
                                          bool doFallTimePattern, bool doPreEmphasisHeight,
                                          bool doPreEmphasisWidth, bool doDeEmphasisHeight,
                                          bool doDeEmphasisWidth, bool doNoise, string RestrictRiseTime, string RestrictFallTime);


        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIConfigurePam4Measurements")]
        private static extern bool ConfigurePam4Measurements(byte instance,
            bool doVerticalEyeHeights, bool doHorizontalEyeWidths,
            bool doVerticalEyeClosure, bool doVerticalEyeHeights_By_BER, bool doHorizontalEyeWidths_By_BER);


        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIConfigureMaskMeasurements")]
        private static extern bool ConfigureMaskMeasurements(byte instance,
                                      bool doMaxMargin, double targetFailingPointsHitRatio,
                                      int maskMarginPercentage, double x1, double x2, double x3, double x4, double x5, double x6,
                                      bool maskYAsPercentage, double y0, double y1, double y2, double y3, double y4, double y5, double y6);

        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIConfigureFilters")]
        private static extern bool ConfigureFilters(byte instance, int ptbResolution, string filtersFile);

        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APICaptureAndMeasure")]
        private static extern bool CaptureAndMeasure(byte instance, int chIndex, int acquisitionCountInput, int accumulateNumPoints);


        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIGetResults")]
        private static extern bool GetResults(byte instance, double[] results);


        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIGetResultsMax")]
        private static extern bool GetResultsMax(byte instance, double[] results);
        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIGetResultsMin")]
        private static extern bool GetResultsMin(byte instance, double[] results);
        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIGetResultsStd")]
        private static extern bool GetResultsStd(byte instance, double[] results);
        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIGetResultsLatest")]
        private static extern bool GetResultsLatest(byte instance, double[] results);



        [DllImport(dllPath, EntryPoint = "APIReadPatternData")]
        private static extern int ReadPatternData(byte instance, double[] xEyeValues, double[] yEyeValues, int size);

        [DllImport(dllPath, EntryPoint = "APIReadEyeData")]
        private static extern int ReadEyeData(byte instance, double[] xEyeValues, double[] yEyeValues, int size);

        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIWriteScopeCalibration")]
        private static extern bool WriteScopeCalibration(byte instance, string IP, int chIndex, bool isOptical, int opticalWaveLength, double A, double B);

        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIReadScopeCalibration")]
        private static extern bool ReadScopeCalibration(byte instance, string IP, int chIndex, bool isOptical, int opticalWaveLength, ref double A, ref double B);

        [return: MarshalAs(UnmanagedType.I1)]
        [DllImport(dllPath, EntryPoint = "APIAccessBoardRegister")]
        private static extern bool AccessBoardRegister(UInt16 Instance, UInt16 Read_Write, UInt16 Index, UInt16 Reg, ref UInt16 Data);

        [DllImport(dllPath, EntryPoint = "APIReadSerialNumber")]
        private static extern int apiReadSerialNumber(byte instance, ref ulong SN);

        /// <summary>
        /// Used to either establish a connection with the DSO over the DSO IP, or to simulate a connection to a DSO by providing Eye.dso data file(s) saved from the GUI
        /// </summary>
        /// <param name="dsoIP">The IP Address of the DSO</param>
        /// <param name="isOptical">True is optical DSO, default value = false</param>
        /// <param name="opticalWaveLength">The wavelength of the optical DSO</param>
        /// <param name="simulationFiles">Array of File names, if it contains files those files will be used for simulation and the provided IP address will not be used</param>
        /// <returns>true when successful</returns>
        public bool APIConnectToDSO(string dsoIP, bool isOptical = false, int opticalWaveLength = 850, string[] simulationFiles = null)
        {
            lock (connectionMonitor)
                return ConnectToDSO(instance, dsoIP, isOptical, opticalWaveLength, simulationFiles, simulationFiles == null ? 0 : simulationFiles.Length);
        }


        public bool APIConfigureDSO(double lineRateInput, double packetCountInput, int patternLengthInput,
                         bool wanderOn, bool precisionTimeBaseOn, bool notmalModeInput, double clockInput,
            double nrzWindowStartInput = 0.2, double nrzWindowsEndInput = 0.8,
            bool configureDSO = false, int samplingFrequencyIndexInput = 0, double attenuatorCompensationDB = 0,
            double totalJitterTargetBER = 1e-12, double deviceIntrinsicJitterFemtoSec = 500, double dJCorrection = 1, double rJrmsCorrection = 1, bool isFilterPjAtSource = false, int percentUIforLevels = 20)
        {
            double samplingFrequency = 0;
            lock (connectionMonitor)
                return ConfigureDSO(instance, lineRateInput, packetCountInput, patternLengthInput,
                                 wanderOn, precisionTimeBaseOn, notmalModeInput, clockInput,
                                 nrzWindowStartInput, nrzWindowsEndInput,
                                 configureDSO, samplingFrequencyIndexInput, attenuatorCompensationDB,
                                 totalJitterTargetBER, deviceIntrinsicJitterFemtoSec, dJCorrection, rJrmsCorrection, isFilterPjAtSource, percentUIforLevels, ref samplingFrequency, (int)SignalCoding.NRZ);
        }

        /// <summary>
        /// Configure the DSO parameters
        /// </summary>
        /// <param name="lineRateInput">Line Rate in Gbps</param>
        /// <param name="packetCountInput">Integer number of packets. The points acquired will equal this parameter x 512 for pattern graph, which are duplicated resulting in packet count x1024 for eye graph</param>
        /// <param name="patternLengthInput">The number of bits in your input pattern</param>
        /// <param name="wanderOn">Boolean to enable wander compensation. Set to false by default</param>
        /// <param name="precisionTimeBaseOn">Boolean to enable Precision Time Base (PTB). PTB will ensure the pattern signal is equally spaced in time, allowing functions such as filters. It also reduces RJ and reduces points used in all operations</param>
        /// <param name="notmalModeInput">True to use Normal Mode. Always set to true. Other modes are internal for Multilane for debugging purposes</param>
        /// <param name="clockInput">The frequency of the input clock in MHz</param>
        /// <param name="nrzWindowStartInput">The NRZ window start, as a decimal from 0 to 1, choose 0.2 for 20 %</param>
        /// <param name="nrzWindowsEndInput">The NRZ window end, as a decimal from 0 to 1, choose 0.8 for 80 %</param>
        /// <param name="configureDSO">Boolean, when set to true, configures the sampling frequency of the DSO and obtains a lock. If there is already a lock and DSO previously had its sampling frequency configured there is no need to repeat</param>
        /// <param name="samplingFrequencyIndexInput">Set to 0 by default. if facing issues at selected sampling frequency, try a higher number, such as 2 or 3. If you choose 2, it will skip the first sampling frequency it finds and goes with a second that might work better</param>
        /// <param name="attenuatorCompensationDB">dB values of any attenuator conencted to the DSO. When configured the DSO software will compensate for this loss. 
        /// 6 dB will double DSO output in case of electrical signal
        /// 3 dB will double DSO output in case of optical signal</param>
        /// <param name="totalJitterTargetBER">Example: 1e-5 for a target TJ BER of 10^-5</param>
        /// <param name="deviceIntrinsicJitterFemtoSec">Default to 0. in Femto-Seconds, when set to 300 fs the device's jitter will be subtracted from the measured jitter.</param>
        /// <returns>true when successful</returns>
        public bool APIConfigureDSO(double lineRateInput, double packetCountInput, int patternLengthInput,
                         bool wanderOn, bool precisionTimeBaseOn, bool notmalModeInput, double clockInput,
            double nrzWindowStartInput, double nrzWindowsEndInput,
            bool configureDSO, int samplingFrequencyIndexInput, double attenuatorCompensationDB,
            double totalJitterTargetBER, double deviceIntrinsicJitterFemtoSec, double dJCorrection, double rJrmsCorrection, bool isFilterPjAtSource, int percentUIforLevels, ref double samplingFrequency, SignalCoding signalCoding)
        {
            lock (connectionMonitor)
                return ConfigureDSO(instance, lineRateInput, packetCountInput, patternLengthInput,
                                 wanderOn, precisionTimeBaseOn, notmalModeInput, clockInput,
                                 nrzWindowStartInput, nrzWindowsEndInput,
                                 configureDSO, samplingFrequencyIndexInput, attenuatorCompensationDB,
                                 totalJitterTargetBER, deviceIntrinsicJitterFemtoSec, dJCorrection, rJrmsCorrection, isFilterPjAtSource, percentUIforLevels, ref samplingFrequency, (int)signalCoding);
        }

        /// <summary>
        /// Configures the behaviour of the API
        /// </summary>
        /// <param name="saveConfig">Saved the configuration used in this path</param>
        /// <param name="saveData">Saveds the data and measurements captured in this path if what follows is true</param>
        /// <param name="SaveStatistics">True to save statistics</param>
        /// <param name="saveDataFlag">True to save raw data</param>
        /// <param name="statisticsPrint">True to output statistics to the standard output (cout<<)</param>
        /// <param name="eyePrint">True to output low-res eye shape to the standard output (cout<<)</param>
        /// <returns>true when successful</returns>
        public bool APIConfigureApplication(string saveConfig, string saveData, bool SaveStatistics, bool saveDataFlag, bool statisticsPrint, bool eyePrint)
        {
            lock (connectionMonitor)
                return ConfigureApplication(instance, saveConfig, saveData, SaveStatistics, saveDataFlag, statisticsPrint, eyePrint);
        }

        /// <summary>
        /// Changes which eye measurements are enabled
        /// </summary>
        /// <param name="doTopMeasurements">Enables measuring top</param>
        /// <param name="doBaseMeasurements">Enables measuring base</param>
        /// <param name="doMin">Enables measuring min</param>
        /// <param name="doMax">Enables measuring max</param>
        /// <param name="doRiseTime">Enables measuring rise time</param>
        /// <param name="doFallTime">Enables measuring fall time</param>
        /// <param name="doPeakToPeak">Enables measuring peak-to-peak amplitude</param>
        /// <param name="doEyeAmplitude">Enables measuring eye amplitude</param>
        /// <param name="doEyeHeight">Enables measuring eye height</param>
        /// <param name="doEyeWidth">Enables measuring eye width</param>
        /// <param name="doCrossingPercentage">Enables measuring crossing</param>
        /// <param name="doJitter">Enables measuring peak-to-peak jitter</param>
        /// <param name="doSNR">Enables measuring total jitter rms from eye diagram (this is not RJ)</param>
        /// <returns>true when successful</returns>
        public bool APIConfigureEyeMeasurements(bool doTopMeasurements, bool doBaseMeasurements, bool doMin,
                                      bool doMax, bool doRiseTime, bool doFallTime, bool doPeakToPeak,
                                      bool doEyeAmplitude, bool doEyeHeight, bool doEyeWidth, bool doCrossingPercentage, bool doJitter, bool doSNR, bool doVEC)
        {
            lock (connectionMonitor)
                return ConfigureEyeMeasurements(instance, doTopMeasurements, doBaseMeasurements, doMin,
                                              doMax, doRiseTime, doFallTime, doPeakToPeak,
                                              doEyeAmplitude, doEyeHeight, doEyeWidth, doCrossingPercentage, doJitter, doSNR, doVEC);
        }

        /// <summary>
        /// Changes which pattern measurements are enabled
        /// </summary>
        /// <param name="doRJ">Enables measuring random jitter</param>
        /// <param name="doDJ">Enables measuring deterministic jitter</param>
        /// <param name="doRiseTimePatterm">Enables measuring rise time at the pattern</param>
        /// <param name="doFallTimePattern">Enables measuring fall time at the pattern</param>
        /// <param name="doPreEmphasisHeight">Enables measuring preEmphasis height</param>
        /// <param name="doPreEmphasisWidth">Enables measuring preEmphasis width</param>
        /// <param name="doDeEmphasisHeight">Enables measuring deEmphasis height</param>
        /// <param name="doDeEmphasisWidth">Enables measuring deEmphasis width</param>
        /// <returns>true when successful</returns>
        public bool APIConfigurePatternMeasurements(bool doRJ, bool doDJ, bool doRiseTimePatterm,
                                          bool doFallTimePattern, bool doPreEmphasisHeight,
                                          bool doPreEmphasisWidth, bool doDeEmphasisHeight,
                                          bool doDeEmphasisWidth, bool doNoise, string RestrictRiseTime, string RestrictFallTime)
        {

            lock (connectionMonitor)
                return (ConfigurePatternMeasurements(instance, doRJ, doDJ, doRiseTimePatterm,
                                             doFallTimePattern, doPreEmphasisHeight, doPreEmphasisWidth, doDeEmphasisHeight, doDeEmphasisWidth, doNoise, RestrictRiseTime, RestrictFallTime));
        }

        /// <summary>
        /// Configures and Activates Mask Measurements
        /// </summary>
        /// <param name="doMaxMargin">True to determine maximum mask margin for the selected target failing points, false to determine number of failing points for the selected mask margin</param>
        /// <param name="targetFailingPoints">The number of allowed failing points for the mask margin</param>
        /// <param name="maskMarginPercentage">The mask margin percentage to use when doMaxMargin = false to determine number of failing points instead</param>
        /// <param name="x1">X1 Parameter as per DSO GUI</param>
        /// <param name="x2">X2 Parameter as per DSO GUI</param>
        /// <param name="x3">X3 Parameter as per DSO GUI</param>
        /// <param name="x4">X4 Parameter as per DSO GUI</param>
        /// <param name="x5">X5 Parameter as per DSO GUI</param>
        /// <param name="x6">X6 Parameter as per DSO GUI</param>
        /// <param name="maskYAsPercentage">True to calculate Y parameters of the mask as a percentage of the Eye Amplitude. In this case Y parameters will be centered at 0.5
        /// False to use the provided Y parameters as-is where the actual expected amplitudes are defined next</param>
        /// <param name="y0">Y0 Parameter as per DSO GUI, usually 0.5 when Y is defined as percentages. Usually 0 in case of electrical masks, or > 0 in case of optical masks</param>
        /// <param name="y1">Y1 Parameter as per DSO GUI</param>
        /// <param name="y2">Y2 Parameter as per DSO GUI</param>
        /// <param name="y3">Y3 Parameter as per DSO GUI</param>
        /// <param name="y4">Y4 Parameter as per DSO GUI</param>
        /// <param name="y5">Y5 Parameter as per DSO GUI</param>
        /// <param name="y6">Y6 Parameter as per DSO GUI</param>
        /// <returns>true when successful</returns>
        public bool APIConfigureMaskMeasurements(bool doMaxMargin, double targetFailingPointsHitRatio,
                                      int maskMarginPercentage, double x1, double x2, double x3, double x4, double x5, double x6,
                                      bool maskYAsPercentage, double y0, double y1, double y2, double y3, double y4, double y5, double y6)
        {
            lock (connectionMonitor)
                return (ConfigureMaskMeasurements(instance, doMaxMargin, targetFailingPointsHitRatio, maskMarginPercentage, x1, x2, x3, x4, x5, x6, maskYAsPercentage, y0, y1, y2, y3, y4, y5, y6));
        }

        /// <summary>
        /// Used to configure and enable PTB + Filters
        /// </summary>
        /// <param name="ptbResolution">The PTB resolution, equivalent to the number of points to sample per bit</param>
        /// <param name="filtersFile">The path of the filter file, saved from the DSO GUI</param>
        /// <returns>true when successful</returns>
        public bool APIConfigureFilters(int ptbResolution, string filtersFile)
        {
            lock (connectionMonitor)
                return ConfigureFilters(instance, ptbResolution, filtersFile);
        }

        /// <summary>
        /// Used to execute the capturing of one or multiple acquisitions, along with the related activated measurements
        /// </summary>
        /// <param name="chIndex">The channel on which to execute, 0 by default</param>
        /// <param name="acquisitionCountInput">Number of acquisitions to execute, the measurement results will be the average, default 1</param>
        /// <param name="accumulateNumPoints">Number of points to accumulate per acquisition, default 0. 
        /// If zero then all the points of a single acquisition will be taken.
        /// If more than the number of points per acquisition for the eye (1024 x Number of Packets), multiple acquisitions will be treated as one acquisition an will accumulate data until the required number of points is reached
        /// If less than the number of points per acquisition, extra points will be truncated randomly</param>
        /// <returns>true when successful</returns>
        public bool APICaptureAndMeasure(int chIndex, int acquisitionCountInput, int accumulateNumPoints = 0)
        {
            bool result = false;

            lock (connectionMonitor)
                result = (CaptureAndMeasure(instance, chIndex, acquisitionCountInput, accumulateNumPoints));

            if (!result)
            {
                //throw new Exception("Invalid data captured, verify correct Clock settings");
            }
            return result;
        }

        /// <summary>
        /// Returns measurement results 
        /// </summary>
        /// <param name="results">The returned results as csv line
        /// </param>
        /// <returns>true when successful</returns>
        public bool APIGetResults(ref double[] results)
        {
            lock (connectionMonitor)
                return GetResults(instance, results);
        }


        public bool APIGetResults(ref Dictionary<E_MEAS, double> results, StatisticResult statistic = StatisticResult.Average)
        {
            E_MEAS[] enumValues = (E_MEAS[])Enum.GetValues(typeof(E_MEAS));
            double[] apiResults = new double[enumValues.Length];
            bool result = false;

            switch (statistic)
            {
                case StatisticResult.Latest:
                    result = GetResultsLatest(instance, apiResults);
                    break;
                case StatisticResult.Min:
                    result = GetResultsMin(instance, apiResults);
                    break;
                case StatisticResult.Max:
                    result = GetResultsMax(instance, apiResults);
                    break;
                case StatisticResult.StandardDeviation:
                    result = GetResultsStd(instance, apiResults);
                    break;
                default:
                    result = GetResults(instance, apiResults);
                    break;
            }

            results = new Dictionary<E_MEAS, double>();
            for (int i = 0; i < enumValues.Length; i++)
            {
                results[enumValues[i]] = apiResults[i];
            }
            return result;
        }

        /// <summary>
        /// Returns the x & y values needed to draw the eye graph
        /// </summary>
        /// <param name="xValue"></param>
        /// <param name="yValues"></param>
        /// <returns>Number of points</returns>
        public int APIReadEyeData(ref double[] xValue, ref double[] yValues)
        {
            lock (connectionMonitor)
                return ReadEyeData(instance, xValue, yValues, xValue.Length);
        }

        /// <summary>
        /// Returns the x & y values needed to draw the pattern graph
        /// </summary>
        /// <param name="xValue"></param>
        /// <param name="yValues"></param>
        /// <returns>Number of points</returns>
        public int APIReadPatternData(ref double[] xValue, ref double[] yValues)
        {
            lock (connectionMonitor)
                return ReadPatternData(instance, xValue, yValues, xValue.Length);
        }

        /// <summary>
        /// Writes scope calibrations. To be used by Multilane only.
        /// </summary>
        /// <param name="IP">IP address of board to write the calibrations to</param>
        /// <param name="chIndex">the index of the channel for which the calibrations apply</param>
        /// <param name="isOptical">true for optical device</param>
        /// <param name="opticalWaveLength">wavelength in case optical device</param>
        /// <param name="A">The A Calibration Factor</param>
        /// <param name="B">The B Calibration Factor</param>
        /// <returns>true when successful</returns>
        public bool APIWriteScopeCalibration(string IP, int chIndex, bool isOptical, int opticalWaveLength, double A, double B)
        {
            lock (connectionMonitor)
                return WriteScopeCalibration(instance, IP, chIndex, isOptical, opticalWaveLength, A, B);
        }

        /// <summary>
        /// Reads the scope calibrations. To be used by Multilane only.
        /// </summary>
        /// <param name="IP">IP address of board to write the calibrations to</param>
        /// <param name="chIndex">the index of the channel for which the calibrations apply</param>
        /// <param name="isOptical">true for optical device</param>
        /// <param name="opticalWaveLength">wavelength in case optical device</param>
        /// <param name="A">The A Calibration Factor</param>
        /// <param name="B">The B Calibration Factor</param>
        /// <returns>true when successful</returns>
        public bool APIReadScopeCalibration(string IP, int chIndex, bool isOptical, int opticalWaveLength, ref double A, ref double B)
        {
            lock (connectionMonitor)
                return ReadScopeCalibration(instance, IP, chIndex, isOptical, opticalWaveLength, ref A, ref B);
        }


        public string APIReadSerialNumber()
        {
            lock (connectionMonitor)
            {
                UInt64 SN = 0;
                int status = apiReadSerialNumber(instance, ref SN);
                return SN.ToString("X");
            }
        }

        public struct BoardState
        {
            public int BoardId;
            public int Temperature;
            public bool LOL_SiLab;
            public bool LOS_Clk;
            public bool IsUsingNewSiLab;
            public double FirmwareRev;
        }

        Dictionary<DateTime, BoardState> BoardStateCache = new Dictionary<DateTime, BoardState>();
        TimeSpan BoardStateCacheTime = new TimeSpan(0, 0, 3);
        public BoardState GetBoardState()
        {
            BoardState[] temp = BoardStateCache
                .Where(c => DateTime.Now.Subtract(c.Key) < BoardStateCacheTime)
                .Select(c => c.Value).ToArray();

            if (temp.Length > 0)
                return temp[0];
            else
            {
                BoardState state = new BoardState();

                GetBoardIdAndSiLab(ref state.BoardId, ref state.IsUsingNewSiLab);
                GetLocksStatus(ref state.LOS_Clk, ref state.LOL_SiLab);
                state.Temperature = GetTemperature();
                state.FirmwareRev = GetFirmwareRev();

                BoardStateCache = new Dictionary<DateTime, BoardState>();
                BoardStateCache[DateTime.Now] = state;
                return state;
            }
        }

        public int GetTemperature()
        {
            ushort degrees = 0;
            lock (connectionMonitor)
                if (AccessBoardRegister(instance, (ushort)Operation.Read, (ushort)0, (ushort)E_DVT.ML4016_TempSense, ref degrees))
                    return (int)degrees;
                else return 0;
        }

        public void GetLocksStatus(ref bool LOL_SiLab, ref bool LOS_Clk)
        {
            ushort portC = 0;
            lock (connectionMonitor)
                if (AccessBoardRegister(instance, (ushort)Operation.Read, (ushort)2, (ushort)E_Clock.Clock_Status, ref portC))
                {
                    LOS_Clk = true;

                    LOL_SiLab = (portC & 0x01) == 1;

                    //int LOS = (portC & 0x0010) >> 4; // 4th bit
                    //LOS_Clk = LOS == 0;
                    //int LOL = (portC & 0x2000) >> 13; // 13th bit
                    //LOL_SiLab = LOL == 0;
                }
                else
                {
                    LOS_Clk = false;
                    LOL_SiLab = false;
                }
        }

        public void GetBoardIdAndSiLab(ref int BoardId, ref bool IsUsingNewSiLab)
        {
            ushort data = 0;
            lock (connectionMonitor)
                if (AccessBoardRegister(instance, (ushort)Operation.Read, (ushort)0, (ushort)E_DVT.Board_ID, ref data))
                {
                    IsUsingNewSiLab = data < 10000; // Currently assumes for all DSOs Board IDs are saved as 2 concatenated HEX resulting in numbers > 10000
                    // May need rethinking, but this solution seems for now more future-proof compared to having a condition for every board type
                    // The new SiLab is for now only used by the 4009A which contains both a BERT and Scope and returns 4009 like all BERTs, < 10000

                    if (IsUsingNewSiLab)
                        BoardId = data;
                    else BoardId = ((data >> 8) * 100 + (data & 0xff));
                }
                else
                {
                    IsUsingNewSiLab = false;
                    BoardId = -1;
                }
        }

        public double GetFirmwareRev()
        {
            ushort regData = 0;
            lock (connectionMonitor)
                if (AccessBoardRegister(instance, (ushort)Operation.Read, (ushort)0, (ushort)E_DVT.DevFirmware_REv, ref regData))
                    return regData / 10.0;
                else return 0;
        }

        public static string Get_ML_DSO_API_Revision()
        {
            string originalFilename = FileVersionInfo.GetVersionInfo(dllPath).OriginalFilename;

            return originalFilename;
        }
    }
}